#include "main_window.h"
#include "ui_main_window.h"
#include <QDir>
#include <QStringList>
#include <QDateTime>

#include "mil1553ud_regs.h"
#include "channel_finder.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    connect(this, &MainWindow::signal_writeInTerminal, this, [this](QString text)->void {
        ui->listWidgetTerminal->addItem(QString("%1: %2")
                                        .arg(QDateTime::currentDateTime().toString("hh:mm:ss.zzz"))
                                        .arg(text)
                                        );
    });
    connect(ui->pushButtonClear, &QPushButton::clicked, ui->listWidgetTerminal, &QListWidget::clear);
    //
    // вкладка тест драйвера - инициализация
    //
    connect(ui->pushButton_startDriverTest, &QPushButton::clicked, this, &MainWindow::slot_driverTest);
    connect(ui->pushButton_startDriverTest, &QPushButton::clicked, this, [this](void)->void {
        ui->pushButton_startDriverTest->setEnabled(false);
        ui->pushButton_stopDriverTest->setEnabled(true);
    });
    connect(ui->pushButton_stopDriverTest, &QPushButton::clicked, this, [this](void)->void {
        emit signal_writeInTerminal(QString("Стоп!"));
        ui->pushButton_startDriverTest->setEnabled(true);
        ui->pushButton_stopDriverTest->setEnabled(false);
    });
    //
    // вкладка простой тест - инициализация
    //
    channelTimerBC = new QTimer(this);
    connect(channelTimerBC, &QTimer::timeout, this, &MainWindow::slot_timerDMA);
    channelTimerRT = new QTimer(this);
    connect(channelTimerRT, &QTimer::timeout, this, &MainWindow::slot_timerDMA_RT);

    connect(ui->pushButton_initBC, &QPushButton::clicked, this, &MainWindow::slot_initBC);
    connect(ui->pushButton_initRT, &QPushButton::clicked, this, &MainWindow::slot_initRT);
    // Управление состоянием кнопок
    connect(ui->pushButton_Stop, &QPushButton::clicked, this, [this](void)->void {
        emit signal_writeInTerminal(QString("Стоп!"));
        ui->pushButton_initBC->setEnabled(true);
        ui->pushButton_initRT->setEnabled(true);
        ui->pushButton_Stop->setEnabled(false);
        if (channelTimerBC != NULL)
            channelTimerBC->stop();
        if (channelTimerRT != NULL)
            channelTimerRT->stop();
    });
    auto slot_enableButtons = [this](void)->void {
        ui->pushButton_initBC->setEnabled(false);
        ui->pushButton_initRT->setEnabled(false);
        ui->pushButton_Stop->setEnabled(true);
    };
    connect(ui->pushButton_initBC, &QPushButton::clicked, this, slot_enableButtons);
    connect(ui->pushButton_initRT, &QPushButton::clicked, this, slot_enableButtons);
    connect(ui->pushButtonUpdateListChannels, &QPushButton::clicked, this, [this](void) -> void {
        ui->comboBoxChannels->clear();
        QVector<ChannelFinder::ChannelInfo> channels = ChannelFinder::getChannelInfoList();
        foreach (ChannelFinder::ChannelInfo channel, channels) {
            ui->comboBoxChannels->addItem(channel.toString());
        }
    });
    connect(ui->comboBoxChannels, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, [this](int index) -> void {
        if (index < 0) {
            ui->groupBoxDriverTest->setEnabled(false);
            ui->groupBoxSimpleTest->setEnabled(false);
        } else {
            ui->groupBoxDriverTest->setEnabled(true);
            ui->groupBoxSimpleTest->setEnabled(true);
        }
    });
}

MainWindow::~MainWindow()
{
    delete ui;
}

//
// вкладка тест драйвера - реализация
//

/// \brief закодированная дата и версия
///  31..28 - major_ver; 27..24 - minor_ver; 23..16 - day; 15..8 - month; 7..0 - year;

void MainWindow::slot_driverTest()
{
    int index = ui->comboBoxChannels->currentIndex();
    if (index < 0)
        return;
    QString nameChannel = ChannelFinder::getChannelList().at(index);
    emit signal_writeInTerminal(QString(" - ТЕСТ ДРАЙВЕРА - '%1' -").arg(nameChannel).toLocal8Bit().constData());

    this->chDevice = open(nameChannel.toLatin1().constData(), O_RDWR);
    if (this->chDevice <= 0) {
        emit signal_writeInTerminal("Символьное устройство канала не найдено!");
        return;
    }
    unsigned int version; // версия драйвера
    ioctl(this->chDevice, IOCTL_VERSION_DRIVER, &version);
    VERSION info;   // информация о драйвере
    ioctl(this->chDevice, IOCTL_VERSION, &info);
    PCI_LOCATION pciLoc; // pci локация
    ioctl(this->chDevice, IOCTL_PCI_LOCATION, &pciLoc);
    emit signal_writeInTerminal(QString("Дата драйвера: %1.%2.%3; Версия драйвера: %4.%5;")
                                .arg(QString::number( (version >> 16) & 0xFF))
                                .arg(QString::number( (version >> 8) & 0xFF))
                                .arg(QString::number( (version >> 0) & 0xFF))
                                .arg(QString::number( (version >> 28) & 0xF))
                                .arg(QString::number( (version >> 24) & 0xF))
                                );
    emit signal_writeInTerminal(QString("DevId: 0x%1; VenId: 0x%2; Type: 0x%3; RevId: 0x%4;")
                                .arg(QString::number(info.device_id, 16))
                                .arg(QString::number(info.vendor_id, 16))
                                .arg(QString::number(info.type, 16))
                                .arg(QString::number(info.revision, 16))
                                );
    emit signal_writeInTerminal(QString("PCI локация: %1;")
                                .arg(QString::fromLatin1(pciLoc.name, SIZE_OF_PCI_LOC))
                                );
}

//
// вкладка простенький тест - реализация
//

void MainWindow::slot_initBC()
{
    auto switchOffChannel = [this](void)->void {
        // выключаем канал
        unsigned int chOn = MIL_DEV_CHANNEL_OFF;
        ioctl(this->chDevice, IOCTL_DEV_CHANNEL_WORK, &chOn);
        // сбрасываем ДМА
        ioctl(this->chDevice, IOCTL_DISABLE_DMA);
        ioctl(this->chDevice, IOCTL_CLEAR_DMA);
    };
    auto initInterruptBc = [this](void)->void {
        // включаем прерывания
        INTERRUPT_MAN iman;
        iman.int_bc = INTERRUPT_MAN_ON;
        ioctl(this->chDevice, IOCTL_MANAGE_INTERRUPT, &iman);
    };
    auto initChannel = [this](void)->void {
        // выбор режима
        unsigned int modeBus = MIL_MODE_BUS_CONTR; // режим КШ
        ioctl(this->chDevice, IOCTL_SWITCH_MODE, &modeBus);
        // выбор шины
        unsigned int bus = MIL_BUS_A_EN | MIL_BUS_B_EN; // шины А и Б
        ioctl(this->chDevice, IOCTL_SWITCH_BUS, &bus);
    };
    auto readRegister = [this](unsigned long regAdr, QString name)->void {
        SADDR_DATA dataAdr;     // регистр
        dataAdr.daddr = regAdr;
        ioctl(this->chDevice, IOCTL_READ_REG, &dataAdr);
        emit signal_writeInTerminal(QString("%1 = %2b")
                                    .arg(name)
                                    .arg(QString::number(dataAdr.data, 2))
                                    );
    };
//    auto writeRegister =[this](unsigned long adr, unsigned int data) -> void {
//        SADDR_DATA dataAdr;     // регистр
//        dataAdr.daddr = adr;
//        dataAdr.data = data;
//        ioctl(this->chDevice, IOCTL_WRITE_REG, &dataAdr);
//    };
    auto writeMicroProgram = [this](void)->void {
        // составляем микропрограмму
        BC_RAM_BLOCK* block_instruction;
        // формируем инструкции
        block_instruction = mil1553ud_newBcRamBlock(MIL_BC_RAM_TYPE_INSTRUCTION, 6, 0);
        block_instruction->dwords[0] = INSTRUCTION(XEQ, ALWAYS, 0);
        block_instruction->dwords[1] = INSTRUCTION(XEQ, ALWAYS, 2);
        block_instruction->dwords[2] = INSTRUCTION(IRQ, 1, 0);
        block_instruction->dwords[3] = INSTRUCTION(XEQ, ALWAYS, 0);
        block_instruction->dwords[4] = INSTRUCTION(XEQ, ALWAYS, 2);
        block_instruction->dwords[5] = INSTRUCTION(HLT, ALWAYS, 0);
        // формируем операции
        BC_RAM_BLOCK* block_operation;
        block_operation = mil1553ud_newBcRamBlock(MIL_BC_RAM_TYPE_OPERATION, 8, 0);
        unsigned int adr = ui->spinBox->value();
        emit signal_writeInTerminal(QString("adr = %1").arg(QString::number(adr)));
        block_operation->dwords[0] = OPERATION_ODD_PART(0x1, 0);
        block_operation->dwords[1] = OPERATION_EVEN_PART( COMMAND_WORD(adr, 0, 1, 16), 0);
        block_operation->dwords[2] = OPERATION_ODD_PART(0x2, 0);
        block_operation->dwords[3] = OPERATION_EVEN_PART( COMMAND_WORD(adr, 1, 2, 2), 0);
        block_operation->dwords[4] = OPERATION_ODD_PART(0x1, 0);
        block_operation->dwords[5] = OPERATION_EVEN_PART( COMMAND_WORD(adr, 0, 1, 16), 0);
        block_operation->dwords[6] = OPERATION_ODD_PART(0x2, 0);
        block_operation->dwords[7] = OPERATION_EVEN_PART( COMMAND_WORD(adr, 1, 2, 2), 0);
        // формируем данные
        BC_RAM_BLOCK* block_data;
        block_data = mil1553ud_newBcRamBlock(MIL_BC_RAM_TYPE_DATA, 2, 0);
        block_data->dwords[0] = (0xAAAA<<16) | 0x5555;
        block_data->dwords[1] = (0xAAAA<<16) | 0x5555;
        // записываем микропрограмму
        ioctl(this->chDevice, IOCTL_WRITE_BC_RAM, block_instruction);
        ioctl(this->chDevice, IOCTL_WRITE_BC_RAM, block_operation);
        ioctl(this->chDevice, IOCTL_WRITE_BC_RAM, block_data);
        // освобождаем буферы микропрограммы
        mil1553ud_freeBcRamBlock(block_instruction);
        mil1553ud_freeBcRamBlock(block_operation);
        mil1553ud_freeBcRamBlock(block_data);
    };
    auto switchOnChannel = [this](unsigned int numChannel)->void {
        Q_UNUSED(numChannel)
        // включаем ДМА
        ioctl(this->chDevice, IOCTL_ENABLE_DMA);
        mil1553ud_bcSetStartInstructionAddress(this->chDevice, 0); // стартовая инструкция микропрограммы
        // включаем канал
        unsigned int chOn = MIL_DEV_CHANNEL_ON;
        ioctl(this->chDevice, IOCTL_DEV_CHANNEL_WORK, &chOn);
    };

    int index = ui->comboBoxChannels->currentIndex();
    if (index < 0)
        return;
    QString nameChannel = ChannelFinder::getChannelList().at(index);
    emit signal_writeInTerminal(QString(" - РЕЖИМ КШ - '%1' -").arg(nameChannel).toLocal8Bit().constData());

    this->chDevice = open(nameChannel.toLatin1().constData(), O_RDWR);

    if (this->chDevice <= 0) {
        emit signal_writeInTerminal("Символьное устройство канала не найдено!");
        return;
    }
    unsigned int numChannel = mil1553ud_getNumberChannel(this->chDevice);
    emit signal_writeInTerminal(QString("Канал = %1").arg(QString::number(numChannel+1)));

    switchOffChannel();
    readRegister(MIL_REG(numChannel, CTRL_REG_PCI_CH), "CTRL_REG_PCI_CH"); // читаем регистр CTRL_REG_PCI_CH
    initChannel();
    initInterruptBc();
    writeMicroProgram();
    switchOnChannel(numChannel);

    mil1553ud_bcProgram(this->chDevice, BC_PROGRAM_START); // BCSTRT - старт выполнения программы КШ

    readRegister(MIL_REG(numChannel, BC_CONF_REG_PCI_CH), "BC_CONF_REG_PCI_CH");
    readRegister(MIL_REG(numChannel, CTRL_REG_PCI_CH), "CTRL_REG_PCI_CH"); // читаем регистр CTRL_REG_PCI_CH

    this->chMode = MIL_MODE_BUS_CONTR;
    channelTimerBC->start(2000);   // старт опросного таймер
}

void MainWindow::slot_initRT()
{
    auto switchOffChannel = [this](void)->void {
        // вЫключаем канал
        unsigned int chOn = MIL_DEV_CHANNEL_OFF;
        ioctl(this->chDevice, IOCTL_DEV_CHANNEL_WORK, &chOn);
        ioctl(this->chDevice, IOCTL_DISABLE_DMA);
        ioctl(this->chDevice, IOCTL_CLEAR_DMA);
    };
    auto initChannel = [this](void)->void {
        unsigned int modeBus = MIL_MODE_TERMINAL_DEV;
        unsigned int bus = MIL_BUS_A_EN | MIL_BUS_B_EN;
        unsigned char address = ui->spinBox->value();
        ioctl(this->chDevice, IOCTL_SWITCH_MODE, &modeBus);
        ioctl(this->chDevice, IOCTL_SET_ADDRESS, &address);
        ioctl(this->chDevice, IOCTL_SWITCH_BUS, &bus);
        emit signal_writeInTerminal(QString("Адрес = %1").arg(QString::number(address)));
    };
    auto initSubaddresses = [this](void)->void {
        // на приём
        RT_BUF_MAN_EN rcvEn;
        rcvEn.direction = MIL_RT_BUF_RECEIVE;
        rcvEn.subaddress_mask = 0x7FFFFFFE; // меняем с 1 по 30 подадрес
        rcvEn.action_mask = 0x7FFFFFFE; // включаем на приём с 1 по 30 подадрес
        ioctl(this->chDevice, IOCTL_RT_BUF_MAN_EN, &rcvEn);
        // на передачу
        RT_BUF_MAN_EN trnEn;
        trnEn.direction = MIL_RT_BUF_TRANSMIT;
        trnEn.subaddress_mask = 0x7FFFFFFE; // меняем с 1 по 30 подадрес
        trnEn.action_mask = 0x7FFFFFFE; // включаем на приём с 1 по 30 подадрес
        ioctl(this->chDevice, IOCTL_RT_BUF_MAN_EN, &trnEn);
    };
    auto initSubaddressBuffer = [this](void)->void {
        for (int i = 1; i <= 30; i++) {
            RT_TR_MODE rt_tr_mode;
            rt_tr_mode.mode = MIL_RT_MODE_PROG;
            rt_tr_mode.subaddress = i;
            ioctl(this->chDevice, IOCTL_SET_RT_TR_MODE, &rt_tr_mode);
            RT_TR_BUF_SUBADDR bufSub;
            bufSub.num_buf = MIL_RT_BUF0;
            bufSub.subaddress = i;
            bufSub.data_words[0] = 0x1111;
            bufSub.data_words[1] = 0x2222;
            ioctl(this->chDevice, IOCTL_WR_BLOCK_BUF_SUBADR, &bufSub);

            RT_TR_MODE_BUF rdBuf;
            rdBuf.cmd_buf = MIL_RT_BUF_0_ON_1_OFF;
            rdBuf.subaddress = i;
            ioctl(this->chDevice, IOCTL_SET_RT_TR_BUF_READY, &rdBuf);
        }
    };
    auto initInterrupt = [this](unsigned int numChannel)->void {
        // включаем прерывания
        INTERRUPT_MAN iman;
        iman.int_rt_ren = INTERRUPT_MAN_ON;
        iman.int_rt_ten = INTERRUPT_MAN_ON;
        ioctl(this->chDevice, IOCTL_MANAGE_INTERRUPT, &iman);

        INTERRUPT_SUBADDRESS intSub;
        intSub.rt_int = INTERRUPT_MAN_ON;
        intSub.subaddress_mask = 0x7FFFFFFE;
        ioctl(this->chDevice, IOCTL_MAN_IRQ_SUBADDRESS_TR, &intSub);

        intSub.rt_int = INTERRUPT_MAN_ON;
        intSub.subaddress_mask = 0x7FFFFFFE;
        ioctl(this->chDevice, IOCTL_MAN_IRQ_SUBADDRESS_RCV, &intSub);

        SADDR_DATA_BIT_MASK bitMask;
        bitMask.daddr = MIL_REG(numChannel, RT_INT_REG_CH);
        bitMask.mask = (1<<0) | (1<<1) | (1<<2);
        bitMask.data = (1<<0) | (1<<1) | (1<<2);
        ioctl(this->chDevice, IOCTL_WRITE_REG_BIT_MASK, &bitMask);
    };
    auto switchOnChannel = [this](void)->void {
        ioctl(this->chDevice, IOCTL_ENABLE_DMA);
        // включаем канал
        unsigned int chOn = MIL_DEV_CHANNEL_ON;
        ioctl(this->chDevice, IOCTL_DEV_CHANNEL_WORK, &chOn);
    };

    int index = ui->comboBoxChannels->currentIndex();
    if (index < 0)
        return;
    QString nameChannel = ChannelFinder::getChannelList().at(index);
    emit signal_writeInTerminal(QString(" - РЕЖИМ ОУ - '%1' -").arg(nameChannel).toLocal8Bit().constData());

    this->chDevice = open(nameChannel.toLatin1().constData(), O_RDWR);

    if (this->chDevice <= 0) {
        emit signal_writeInTerminal("Символьное устройство канала не найдено!");
        return;
    }
    unsigned int numChannel = mil1553ud_getNumberChannel(this->chDevice);
    emit signal_writeInTerminal(QString("Канал = %1").arg(QString::number(numChannel+1)));

    switchOffChannel();
    initChannel();
    initSubaddresses();
    initSubaddressBuffer();
    initInterrupt(numChannel);
    switchOnChannel();

    this->chMode = MIL_MODE_TERMINAL_DEV;
    channelTimerRT->start(2000); // старт опросного таймера
}

void MainWindow::slot_timerDMA()
{
    unsigned int err = 0;
    unsigned int countDmaBlocks = mil1553ud_getCountDmaBlocksReadyRead(this->chDevice, &err); // чтение кол-во блоков дма для чтения
    unsigned int countInterrupts = mil1553ud_getCountInterruptBlocksReadyRead(this->chDevice); // чтения кол-во блоков прерываний для четния
    if (countInterrupts) {
        emit signal_writeInTerminal(QString("countInterrupts = %1").arg(QString::number(countInterrupts)));
        INTERRUPT_BLOCK_BUFFER* intBlock = mil1553ud_readAllInterruptBlocksReadyRead(this->chDevice);
        for (unsigned int i = 0; i < intBlock->count_blocks; i++) {
            emit signal_writeInTerminal(QString("freetimer = %1, interrupt_ch = %2b")
                                        .arg(QString::number(intBlock->blocks[i].free_timer_value))
                                        .arg(QString::number(intBlock->blocks[i].interrupt_ch, 2))
                                        );
        }
        mil1553ud_freeInterruptBlock(intBlock);
    }
    if (countDmaBlocks == 0)
        return;
    emit signal_writeInTerminal("timerDMA");
    unsigned int countReadedBlocks = 0;
    DMA_READ_BLOCK* dmaBlockContainer = mil1553ud_readAllDmaBlocksReadyRead(this->chDevice, &countReadedBlocks); // считываем все блоки ДМА готовые для чтения
    if (dmaBlockContainer == NULL) {
        emit signal_writeInTerminal("dmaBlock == NULL");
        return;
    }
    for (unsigned int indexBlock = 0; indexBlock < dmaBlockContainer->countBlocks; indexBlock++) {
        if (ASSERT_DMA_CONTAINER(dmaBlockContainer, indexBlock) == MIL1553UD_FALSE)
            continue;
        unsigned char* block = GET_PTR_DMA_BLOCK(dmaBlockContainer->data, indexBlock); // текущий 128 байтный блок ДМА
        processingDmaBlockBc(block);// разбор блока ДМА КШ
    }
    mil1553ud_freeDmaReadBlock(dmaBlockContainer);
}

void MainWindow::slot_timerDMA_RT()
{
    unsigned int err = 0;
    unsigned int countDmaBlocks = mil1553ud_getCountDmaBlocksReadyRead(this->chDevice, &err); // чтение кол-во блоков дма для чтения
    unsigned int countInterrupts = mil1553ud_getCountInterruptBlocksReadyRead(this->chDevice); // чтения кол-во блоков прерываний для четния
    if (countInterrupts) {
        emit signal_writeInTerminal(QString("countInterrupts = %1").arg(QString::number(countInterrupts)));        
        INTERRUPT_BLOCK_BUFFER* intBlock = mil1553ud_readAllInterruptBlocksReadyRead(this->chDevice);
        for (unsigned int i = 0; i < intBlock->count_blocks; i++) {
            emit signal_writeInTerminal(QString("freetimer = %1, interrupt_ch = %2b")
                                        .arg(QString::number(intBlock->blocks[i].free_timer_value))
                                        .arg(QString::number(intBlock->blocks[i].interrupt_ch, 2))
                                        );
        }
        mil1553ud_freeInterruptBlock(intBlock);
    }
    if (countDmaBlocks == 0)
        return;
    emit signal_writeInTerminal("timerDMA");
    unsigned int countReadedBlocks = 0;
    DMA_READ_BLOCK* dmaBlockContainer = mil1553ud_readAllDmaBlocksReadyRead(this->chDevice, &countReadedBlocks); // считываем все блоки ДМА готовые для чтения
    if (dmaBlockContainer == NULL) {
        emit signal_writeInTerminal("dmaBlock == NULL");
        return;
    }
    for (unsigned int indexBlock = 0; indexBlock < dmaBlockContainer->countBlocks; indexBlock++) {
        if (ASSERT_DMA_CONTAINER(dmaBlockContainer, indexBlock) == MIL1553UD_FALSE)
            continue;
        unsigned char* block = GET_PTR_DMA_BLOCK(dmaBlockContainer->data, indexBlock); // текущий 128 байтный блок ДМА
        processingDmaBlockTd(block);// разбор блока ДМА ОУ
    }
    mil1553ud_freeDmaReadBlock(dmaBlockContainer);
}

void MainWindow::processingDmaBlockTd(unsigned char *block)
{
    unsigned int typeDmaBlock = MIL1553UD_TD_DMA_BLOCK;     // ОУ
    unsigned int isSuccessTransaction = mil1553ud_dmaBlock_isSuccessTransaction(typeDmaBlock, block);
    unsigned int typeTransaction = mil1553ud_dmaBlock_getTypeTransaction(typeDmaBlock, block);
    unsigned int freeTimer = mil1553ud_dmaBlock_getFreeTimer(typeDmaBlock, block);
    unsigned int countServiceWords16 = mil1553ud_dmaBlock_getCountServiceWords16(typeDmaBlock, block);
    unsigned int countDataWords16 = mil1553ud_dmaBlock_getCountDataWords16(typeDmaBlock, block);
    unsigned int subaddress = mil1553ud_dmaBlock_tdModeSubaddress(block);
    unsigned int activeBus = mil1553ud_dmaBlock_getActiveBus(typeDmaBlock, block);
    emit signal_writeInTerminal(
                QString("Транзакция - тип = %1 - %2 - freeTimer: %3 - CntServWords: %4 - CntDataWords: %5 - Subaddress: %6 - Шина: %7")
                .arg(QString::number(typeTransaction))
                .arg( (isSuccessTransaction) ? "нет ошибок" : "ошибка" )
                .arg(QString::number(freeTimer))
                .arg(QString::number(countServiceWords16))
                .arg(QString::number(countDataWords16))
                .arg(QString::number(subaddress))
                .arg( (activeBus == MIL1553_BUS_A) ? "Шина А" : "Шина Б" )
                                );
    if (countDataWords16 > 0) {
        unsigned short* dataWord16 = (unsigned short*)mil1553ud_dmaBlock_getPtrDataPart(block);
        QString dataWordsString = QString("Слова данных:");
        for(unsigned int i = 0; i < countDataWords16; i++) {
            if (i == countDataWords16-1) {
                dataWordsString += QString(" 0x%1;")
                        .arg(QString::number(dataWord16[i], 16));
            } else {
                dataWordsString += QString(" 0x%1,")
                        .arg(QString::number(dataWord16[i], 16));
            }
        }
        emit signal_writeInTerminal(dataWordsString);
    }
}

void MainWindow::processingDmaBlockBc(unsigned char *block)
{
    unsigned int typeDmaBlock = MIL1553UD_BC_DMA_BLOCK; // КШ
    unsigned int isSuccessTransaction = mil1553ud_dmaBlock_isSuccessTransaction(typeDmaBlock, block);
    unsigned int typeTransaction = mil1553ud_dmaBlock_getTypeTransaction(typeDmaBlock, block);
    unsigned int freeTimer = mil1553ud_dmaBlock_getFreeTimer(typeDmaBlock, block);
    unsigned int countServiceWords16 = mil1553ud_dmaBlock_getCountServiceWords16(typeDmaBlock, block);
    unsigned int countDataWords16 = mil1553ud_dmaBlock_getCountDataWords16(typeDmaBlock, block);
    unsigned int activeBus = mil1553ud_dmaBlock_getActiveBus(typeDmaBlock, block);
    emit signal_writeInTerminal(
                QString("Транзакция - тип = %1 - %2 - freeTimer: %3 - CntServWords: %4 - CntDataWords: %5 - Шина: %7")
                .arg(QString::number(typeTransaction))
                .arg( (isSuccessTransaction) ? "нет ошибок" : "ошибка" )
                .arg(QString::number(freeTimer))
                .arg(QString::number(countServiceWords16))
                .arg(QString::number(countDataWords16))
                .arg( (activeBus == MIL1553_BUS_A) ? "Шина А" : "Шина Б" )
                                );
    if (countDataWords16 > 0) {
        unsigned short* dataWord16 = (unsigned short*)mil1553ud_dmaBlock_getPtrDataPart(block);
        QString dataWordsString = QString("Слова данных:");
        for(unsigned int i = 0; i < countDataWords16; i++) {
            if (i == countDataWords16-1) {
                dataWordsString += QString(" 0x%1;")
                        .arg(QString::number(dataWord16[i], 16));
            } else {
                dataWordsString += QString(" 0x%1,")
                        .arg(QString::number(dataWord16[i], 16));
            }
        }
        emit signal_writeInTerminal(dataWordsString);
    }
}

//
// вкладка простенький тест - реализация - конец
//
