#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H

#include <QMainWindow>
#include "mil1553ud_library.h"
#include <QTimer>

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

signals:
    ///
    /// \brief signal_writeInTerminal отправить текст в терминал
    /// \param text текст
    ///
    void signal_writeInTerminal(QString text);
    //
    // вкладка тест драйвера
    //
private slots:
    void slot_driverTest();
    //
    // вкладка простой тест - инициализация
    //
private slots:
    void slot_initBC();
    void slot_initRT();
    void slot_timerDMA();
    void slot_timerDMA_RT();
private:  
    void processingDmaBlockTd(unsigned char *block); // оу
    void processingDmaBlockBc(unsigned char *block); // кш

private:
    ///
    /// \brief DMA_BLOCK_SIZE константа размера блока ДМА
    ///
    const int DMA_BLOCK_SIZE = 128;
    ///
    /// \brief ui ГУИ
    ///
    Ui::MainWindow *ui;
    ///
    /// \brief chDevice дескриптор файла
    ///
    int chDevice;
    ///
    /// \brief chMode режим работы канала
    ///
    int chMode; // 1 - КШ // 2 - ОУ
    ///
    /// \brief channelTimerBC программный таймер КШ
    ///
    QTimer* channelTimerBC = NULL;
    ///
    /// \brief channelTimerRT программный таймер ОУ
    ///
    QTimer* channelTimerRT = NULL;
};

#endif // MAIN_WINDOW_H
