/// \file mil1553ud_library.c
/// \brief библиотка взаимодействия
/// реалитзация функций
#include "mil1553ud_library.h"

unsigned int mil1553ud_getNumberChannel(int fd)
{
    unsigned int numCh = 0;         // номер канала
    if (ioctl(fd, IOCTL_GET_NUMBER_CH, &numCh))
        return LIB_BAD_VALUE;   // ошибка
    else
        return numCh;
}

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ДМА

unsigned int mil1553ud_getCountDmaBlocksReadyRead(int fd, unsigned int* codeError )
{
    unsigned int countBlocks = 0;   // кол-во блоков
    *codeError = ioctl(fd, IOCTL_GET_NBLOCK_RAW_DMA, &countBlocks);
    int code = *codeError ;
    if (code)
        return LIB_BAD_VALUE;   // ошибка
    else
        return countBlocks;
}

DMA_READ_BLOCK* mil1553ud_newDmaReadBlock(unsigned int countDmaBlocks)
{
    DMA_READ_BLOCK* block = (DMA_READ_BLOCK*)malloc(sizeof(DMA_READ_BLOCK));
    block->countBlocks = countDmaBlocks;
    block->length = block->countBlocks * MIL_DMA_BLOCK_SIZE;
    block->data = (unsigned char*)malloc(block->length);
    return block;
}

void mil1553ud_freeDmaReadBlock(DMA_READ_BLOCK *block)
{
    if ( block != NULL ) {
        if (block->data != NULL) {
            free(block->data);
        }
        free(block);
    }
}

DMA_READ_BLOCK* mil1553ud_readAllDmaBlocksReadyRead(int fd, unsigned int* cntBlocks)
{
    DMA_READ_BLOCK* block = NULL;   // контейнер дма блоков
    unsigned int countBlocks = 0;   // кол-во блоков
    unsigned int codeError = 0;
    countBlocks = mil1553ud_getCountDmaBlocksReadyRead(fd, &codeError);
    *cntBlocks = countBlocks;
    if ( countBlocks == 0 ) {
        return block;
    } else {
        if (codeError != 0) {
            return NULL;
        }
        block = mil1553ud_newDmaReadBlock(countBlocks);
        if (ioctl(fd, IOCTL_READ_BLOCKS_RAW_DMA, block)) { // ошибка
            mil1553ud_freeDmaReadBlock(block);
            return NULL;
        } else {
            return block;
        }
    }
}

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ПРЕРЫВАНИЙ

unsigned int mil1553ud_getCountInterruptBlocksReadyRead(int fd)
{
    unsigned int countBlocks = 0;   // кол-во блоков
    if (ioctl(fd, IOCTL_GET_COUNT_INTERRUPT_BLOCKS, &countBlocks))
        return LIB_BAD_VALUE; // ошибка
    else
        return countBlocks;
}

INTERRUPT_BLOCK_BUFFER* mil1553ud_newInterruptBlock(unsigned int countBlocks)
{
    INTERRUPT_BLOCK_BUFFER* intBlock = (INTERRUPT_BLOCK_BUFFER*)malloc(sizeof(INTERRUPT_BLOCK_BUFFER));
    intBlock->count_blocks = countBlocks;
    intBlock->blocks = (struct block_info*)malloc(sizeof(struct block_info)*intBlock->count_blocks);
    return intBlock;
}

void mil1553ud_freeInterruptBlock(INTERRUPT_BLOCK_BUFFER* block)
{
    if ( block != NULL ) {
        if ( block->blocks != NULL ) {
            free(block->blocks);
        }
        free(block);
    }
}

INTERRUPT_BLOCK_BUFFER* mil1553ud_readAllInterruptBlocksReadyRead(int fd)
{
    INTERRUPT_BLOCK_BUFFER* block = NULL;   // контейнер блоков
    unsigned int countBlocks = 0;   // кол-во блоков
    countBlocks = mil1553ud_getCountInterruptBlocksReadyRead(fd);
    if ( countBlocks == 0 ) {
        return block;
    } else {
        block = mil1553ud_newInterruptBlock(countBlocks);
        if (ioctl(fd, IOCTL_READ_INTERRUPT_BLOCKS, block)) { // ошибка
            mil1553ud_freeInterruptBlock(block);
            return NULL;
        } else {
            return block;
        }
    }
}

/// \remark РЕЖИМ КШ

unsigned int mil1553ud_bcProgram(int fd, unsigned int action)
{
    SADDR_DATA_BIT_MASK bitMask;    // битовая маска регистра
    unsigned int numCh = mil1553ud_getNumberChannel(fd);
    bitMask.daddr = MIL_REG(numCh, BC_CONF_REG_PCI_CH);     // адрес регистра
    bitMask.mask = 1 << 15; // изменяем только 15 бит - BCSTRT
    if (action == (unsigned int)BC_PROGRAM_START) {
        bitMask.data = 1 << 15; // BCSTRT в 1
    } else {
        bitMask.data = 0;
    }
    if (ioctl(fd, IOCTL_WRITE_REG_BIT_MASK, &bitMask))
        return LIB_BAD_VALUE; // ошибка
    else
        return LIB_GOOD;
}

unsigned int mil1553ud_bcSetStartInstructionAddress(int fd, unsigned int startNumber)
{
    SADDR_DATA_BIT_MASK bitMask;    // битовая маска регистра
    unsigned int numChannel = mil1553ud_getNumberChannel(fd);
    bitMask.daddr = MIL_REG(numChannel, START_ADR_INSTR_PTR_CH);    // адрес регистра стартовой инструкции
    bitMask.mask = 0xFFF << 16;
    bitMask.data = (startNumber << 16) & (0xFFF << 16);
    if (ioctl(fd, IOCTL_WRITE_REG_BIT_MASK, &bitMask))
        return LIB_BAD_VALUE; // ошибка
    else
        return LIB_GOOD;
}

/// \remark РЕЖИМ КШ - РАБОТА С МИКРОПРОГРАММОЙ КШ

BC_RAM_BLOCK *mil1553ud_newBcRamBlock(unsigned int type, unsigned int sizeInDwords, unsigned int shiftInDwords)
{
    BC_RAM_BLOCK* block = (BC_RAM_BLOCK*)malloc(sizeof(BC_RAM_BLOCK));
    block->type = type;
    block->length = sizeInDwords;
    block->shift = shiftInDwords;
    block->dwords = (unsigned int*)malloc(block->length * DWORD_SIZE);
    return block;
}

void mil1553ud_freeBcRamBlock(BC_RAM_BLOCK *block)
{
    if (block != NULL) {
        if (block->dwords != NULL) {
            free(block->dwords);
        }
        free(block);
    }
}

/// \remark РАСШИФРОВКА БЛОКОВ ДМА

unsigned int mil1553ud_dmaBlock_getTypeTransaction(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 1) & 0xF;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 27) & 0xF;
    }
}

unsigned int mil1553ud_dmaBlock_isSuccessTransaction(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return ((*dword >> 28) & 0x1) ? MIL1553UD_TRUE : MIL1553UD_FALSE;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return ((*dword >> 26) & 0x1) ? MIL1553UD_FALSE : MIL1553UD_TRUE;
    }
}

unsigned int mil1553ud_dmaBlock_getFreeTimer(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 1 * DWORD_SIZE);
        return *dword;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 4 * DWORD_SIZE);
        return *dword;
    }
}

unsigned int mil1553ud_dmaBlock_getCountServiceWords16(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 11) & 0xF;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 11) & 0xF;
    }
}

unsigned int mil1553ud_dmaBlock_getCountDataWords16(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 5) & 0x3F;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 5) & 0x3F;
    }
}

unsigned int mil1553ud_dmaBlock_getActiveBus(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return ( (*dword >> 15) & 0x1 ) ? MIL1553_BUS_B : MIL1553_BUS_A ;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return ( (*dword >> 31) & 0x1 ) ? MIL1553_BUS_B : MIL1553_BUS_A ;
    }
}

unsigned char *mil1553ud_dmaBlock_getPtrDataPart(unsigned char *dmaBlock)
{
    return dmaBlock + MIL_DMA_BLOCK_SIZE/2;
}

unsigned int mil1553ud_dmaBlock_tdModeSubaddress(unsigned char *dmaBlock)
{
    return *((unsigned int*)dmaBlock) & 0x1F;
}


