#!/bin/bash

# Данный скрипт должен быть вызван из репозитория Git

# Вывод ошибки, ожидание клавиши
showError()
{
 >&2 printf "error: $@"
 read -n1 -rsp $'Press any key to continue or Ctrl+C to exit...\n'
}

# Генерация файла проекта по умолчанию
createFile()
{
 case $1 in
  1) # Базовая зависимость, если список отсутствует
  TEMPL=''
  ;;

  2) # Начальный pro-файл согласно группе репозитория, если он отсутствует
  TEMPL='TEMPLATE = lib'

  for PREFIX in applications plugins libs
  do
   if [[ "${PROJECT_GROUP//$PREFIX/}" != "${PROJECT_GROUP}" ]]
   then
    case "$PROJECT_GROUP" in
     applications) TEMPL='TEMPLATE = app'
     ;;
     plugins) TEMPL="$TEMPL\r\nCONFIG += plugin"
     ;;
     libs) TEMPL="$TEMPL\r\nDEFINES += ${PROJECT_NAME^^}_LIBRARY"
     ;;
    esac
   fi
  done

  TEMPL="$TEMPL\r\n! include(../../sources.pri) { error(\"Invalid path to sources.pri\") }"
  ;;

  *) return 1
  ;;
 esac

 printf "$TEMPL" > "$FILE_DEPENDS"
 return $?
}

EXISTING_ITEMS=""

# Функция рекурсивной обработки зависимости
cloneProject()
{
 # Проект для разработки
 # local PRO_CONTENT="";
 local FOUND_ITEMS=""

 # Пока стек зависимостей не пуст, продолжаем
 while [ -n "$PROJECTS" ]
 do
  # Вынимаем запись из стека
  local PROJECT=${PROJECTS%%\:*}
  PROJECTS=${PROJECTS#$PROJECT\:}

  # Игнорируем прежнее тестовое КПО
  if [ "$PROJECT" == "libs/PluginInterface/tests/applications/TestApp" ]
  then
   echo "Ignoring $PROJECT"
   continue
  fi

  # Получаем параметры проекта по его абсолютному пути	
  local PROJECT_NAME=${PROJECT##*/}
  local PROJECT_GROUP=${PROJECT%%/$PROJECT_NAME}
  PROJECT_GROUP=${PROJECT_GROUP##*/}

  echo ----------------------------------------------
  echo Exist: $EXISTING_ITEMS
  echo Queued: $PROJECTS
  echo Processing: "$PROJECT".

  if [ -d "$PROJECT" ]
  then # папка найдена
   echo "Found $PROJECT"
  else # Скачиваем проект, если его папка не найдена
   local REPO_NAME="git@$SERVER_NAME:/$PROJECT.git"
   echo Trying to clone $PROJECT
   if ! git clone -- "$REPO_NAME" "$PROJECT"
   then

    >&2 printf "error: Non-existing dependency \"$PROJECT\", check files:\n"
    cd "$TOPLEVEL"
    cd ../..

    grep -l -r "$PROJECT" --include depends.list | while read DEPFILE
    do
     >&2 printf "error: $DEPFILE\n"
    done

    showError "Could not clone \"$REPO_NAME\" in \"$PROJECT\""
    return 1
   fi
   echo "Cloned $PROJECT successfully"
  fi

  EXISTING_ITEMS="${PROJECT}:${EXISTING_ITEMS}" # Сохраняем найденное
  FOUND_ITEMS=""

  # Для PluginInterface исключение - у него нет зависимостей,
  # из него в корень развёрнутого репозитория копируются файлы sources.pri и DevProject.pri
  if [ "$PROJECT" != "libs/PluginInterface" ]
  then # У остальных проектов проверяем, созданы ли необходимые файлы
  for FILE_INDEX in 1 2
  do
   case $FILE_INDEX in
    1) FILE_DEPENDS="$PROJECT/depends.list"
    ;;

    2) FILE_DEPENDS="$PROJECT/$PROJECT_NAME.pro"
    ;;
   esac

   if ! [ -f "$FILE_DEPENDS" ] # При отсутствии необходимого файла, создаём заготовку
   then
    echo "Could not find $FILE_DEPENDS. Creating..."
    if ! createFile "$FILE_INDEX"
    then
     showError "Could not write to $FILE_DEPENDS"
      return 1
    fi
    echo "Created $FILE_DEPENDS"
   fi
  done


  # Вся найденная группа зависимостей помещается в стек, но внутри группы порядок сохраняется согласно указанному в файле
  while read ITEM || [ -n "$ITEM" ]
  do
   if [ -n "$ITEM" ] && [ "${ITEM:0:1}" != "#" ] # Комментарии игнорируем
   then
    ITEM="${ITEM#"${ITEM%%[![:space:]]*}"}" # Убираем лидирующие пробелы
    ITEM="${ITEM%"${ITEM##*[![:space:]]}"}" # Убираем замыкающие пробелы
    if [ -n "$ITEM" ] # Игнорируем пустые строки
    then
     if [[ "$EXISTING_ITEMS" == "${EXISTING_ITEMS//${ITEM}://}" ]] # Такой зависимости ещё нет
     then
      echo "Found dependency [NEW]: $ITEM"
      FOUND_ITEMS+="${ITEM}:" # Добавляем в группу согласно порядку следования в файле
     else
      echo "Found dependency [ OK]: $ITEM"
     fi
    fi
   fi
  done < $PROJECT/depends.list

 PROJECTS="${FOUND_ITEMS}${PROJECTS}" # Всю группу помещаем в стек
 fi

#Добавляем строки в pro-файле, если не содержатся
#local PROJECT_VAR="\nSUBDIRS *= ${PROJECT//\//_}"
#if [ "${PRO_CONTENT//${PROJECT_VAR}//}" == "${PRO_CONTENT}" ]
#then
#	PRO_CONTENT+=${PROJECT_VAR}
#fi

 done

# PRO_CONTENT="`printf \"${PRO_CONTENT}\" | sort`"

PRO_FILE_NAME="DevProject"

local CUR_DIR_NAME="`pwd`"
CUR_DIR_NAME="${CUR_DIR_NAME##*/}"
CUR_DIR_NAME="${CUR_DIR_NAME// //}"

if [ -n "$CUR_DIR_NAME" ]
then
 # Добавляем название текущего каталога, чтобы обеспечить уникальность имён DevProject из соседних каталогов
 PRO_FILE_NAME+="-${EXEC_PROJ_NAME}-${CUR_DIR_NAME}"
fi

PRO_FILE_NAME+=".pro"

if ! printf "$PRO_CONTENT" > "$PRO_FILE_NAME"
then
 showError "Could not write to $PRO_FILE_NAME"
 return 1
fi

return 0
}


# Переходим в корень репозитория git
TOPLEVEL="`git rev-parse --show-toplevel`"

if [ -z "$TOPLEVEL" ]
then
 showError "Project is not in repository!"
 exit 1
fi

cd "$TOPLEVEL"

CUR_DIR_NAME="$(basename $(pwd))"
EXEC_PROJ_NAME="${CUR_DIR_NAME}"

# Определяем параметры собственного проекта
OWN_PROJECT_PATH="$(basename $(dirname $(pwd)))/${CUR_DIR_NAME}"

PRO_CONTENT="TEMPLATE = subdirs\n\
SUBDIRS *= ${OWN_PROJECT_PATH//\//_}\n\
include (DevProject.pri)"

# Подготавливаем стек для рекурсивного клонирования
# Добавляем в стек проект
PROJECTS="$OWN_PROJECT_PATH:"
echo "Recursive cloning \"$OWN_PROJECT_PATH\""

# Получаем название сервера
SERVER_NAME=`git config --get remote.origin.url | sed 's/.*@//g;s/:.*//g'`
if [ -z "$SERVER_NAME" ]
then
 showError "Empty repository name"
 exit 1
fi

cd ../..
find -maxdepth 1 -iname \*pri -exec touch {} \;


# Настраиваем doxygen, если он установлен
if which doxygen &> /dev/null
then
 doxygen -g &> /dev/null

 # Настраиваем dot, если он установлен
 HAVE_DOT="NO"
 if which dot &> /dev/null
 then
  HAVE_DOT="YES"
 fi
 sed -e "s/English$/Russian/g;s/^RECURSIVE.*$/RECURSIVE\t\t = YES/g;s/^HAVE_DOT.*$/HAVE_DOT\t\t = ${HAVE_DOT}/g" -i Doxyfile
fi

echo "Server is $SERVER_NAME"
pwd

# Выполняем рекурсивное клонирование
if ! cloneProject
then
 showError "Error in recursing. Fix the problem and try again later!"
 exit 1
fi

printf "\n\nSUCCESS. Now you should CLEAN, QMAKE, REBUILD and REOPEN your developer project $(pwd)/$PRO_FILE_NAME\n"
exit 0
