# Описание "Системы сборки"
"Система сборки" предназначена для организации исходного кода с целью максимизации его повторного использования 
# Структура каталогов проекта, использующего "Систему сборки", на машине разработчика
У проектов, построенных с помощью "Системы сборки", следующая структура каталогов:
* 'корневая папка'
    * Git
        * applications[Тема]
        * [libs[Тема]]
        * [plugins[Тема]]

Имя корневой папки не имеет значения, "Система сборки" будет размещать промежуточные артефакты построения на этот уровень, по-этому важно чтоб она просто была. В одной папке Git можно вести разработку любого количества приложений, они не будут друг другу мешать.
# Порядок работы
Для того, чтобы развернуть среду разработки ранее созданого приложения необходимо:
1. завести соответствующую папку согласно структуре каталогов проекта. 'корневая папка'/Git/applications[Тема]
2. склонировать в эту папку репозиторий желаемого проекта
3. в склонированном репозитории найти и запустить скрипт dev-configure.sh. Он не только доклонирует недостающие зависимости для данного приложения, но и сгенрирует DevProject-'имя проекта'-Git.pro файл проекта разработки для QtCreator в папке Git
4. открыть файл DevProject-'имя проекта'-Git.pro с помощью QtCreator
5. добавить в make настроек проекта install [-j \`nproc --all\`] (опциональная часть нужна для того, чтобы задействовать все ядра при сборке)
6. запустить qmake у корневого проекта, после выполнения которого закройть QtCreator и открыть DevProject-'имя проекта'-Git.pro снова

Помимо указанных папок в папке Git, "Система сборки" ещё создаст папку build с результатом сборки проектов.

В каждом приложении/библиотеке/плагине есть файл depends.list, который описывает зависимости в формате libs/"имя библиотеки, от которой есть зависимость". Если зависимостей нет, то файл пуст, но файл должен быть

# Для создания нового проекта:
1. На гит сервере создайте репозиторий с именем проекта в группе applications[Тема].
2. Клонируйте новый репозиторий в папку 'корневая папка'/Git/applications[Тема]. появится папка проекта
3. файл main.cpp и все исходники расположите в подпапке /src
4. разархивируйте архив startProject.tar.gz из репозитория developers/buildSystem.git в папку проекта
5. запустите скрипт dev-configure.sh
6. откройте сгенерированный файл 'корневая папка'/Git/DevProject-'имя проекта'-Git.pro и добавьте в make настроек проекта install -j \`nproc --all\`
7. запустите qmake у корневого проекта, после выполнения которого закройте QtCreator и откройте DevProject-'имя проекта'-Git.pro заново

После редактирования какого-либо файла depends.list (приложения или одной из зависимости), необходимо выполнить скрипт dev-configure.sh чтобы он доклонировал новые зависимости, если они ещё не склонированы.

# Для создания библиотеки:
1. На гит сервере создайте репозиторий с именем библиотеки в группе libs[Тема]
2. Клонируйте новый репозиторий в папку 'корневая папка'/Git/libs[Тема]. появится папка библиотеки
3. Добавьте эту библиотеку в depends.list использующего её приложения или другой библиотеки
4. выполните dev-configure.sh использующего эту библиотеку приложения
5. запустите qmake у корневого проекта из папки Git, чтобы библиотека появилась в проекте

Так же, для экспортирования классов и свободных функций, которые имеют скрытую реализацию и которые доступны из вне, необходимо определить макрос
```
#if defined(`имя библиотеки большими буквами`_LIBRARY)
#define `имя библиотеки большими буквами`_API Q_DECL_EXPORT
#else
#define `имя библиотеки большими буквами`_API Q_DECL_IMPORT
#endif
```

Исходники с заголовочниками, которые будут доступны зависимым библиотекам и приложениям, следует распологать в подпапке src/. А исходники с заголовочниками, которые доступны только в рамках этой библиотеки или приложения - в папке src-private. Исходники можно группировать по подпапкам, что увеличит разграничение области видимости. Например: для того, чтобы включить заголовочник src/subfolder/header.h, нужно будет написать #include "subfolder/header.h"
